@@ -74,12 +74,12 @@ def load_input_string(fname):
     return txt
 
 # [hash(#line)?] content [/hash]
-hash_format = re.compile(r'\[([A-Za-z0-9_-]+)(#\d+|)\](.*)\[/\1\]', re.DOTALL)
+hash_format = re.compile(r'\[([A-Za-z0-9_-]+)(#\d+|)\](.*?)\[/\1\]', re.DOTALL)
 # the line number is completely useless, but it now appears
 # in the expected files in 365 repositories, so we need to still
 # parse it - 2021-07-23
 
-def load_output(fname):
+def load_output(fname, should_sort_analyses=False):
     try:
         with open(fname, 'r') as fin:
             ret = {}
@@ -88,6 +88,8 @@ def load_output(fname):
                 content = content_.strip()
                 if not content:
                     print('ERROR: Entry %s in %s was empty!' % (hsh, fname))
+                if should_sort_analyses:
+                    content = sort_analyses(content)
                 l = 0
                 if line:
                     l = int(line[1:])
@@ -114,7 +116,7 @@ def load_gold(fname):
                     if o2:
                         opts.append(o2)
                 if not opts:
-                    print('ERROR: Empty entry %s in %s' % (ident, fname))
+                    print('ERROR: Empty entry %s in %s' % (hsh, fname))
                     continue
                 ret[hsh] = opts
             return ret
@@ -129,6 +131,63 @@ def save_gold(fname, data):
                 fout.write('%s [/option]\n' % ln)
             fout.write('[/%s]\n' % inhash)
 
+apertium_wblank_pat = (r'(?:\[\[' +   # [[
+                       ('(?:' +
+                        r'[^\]\\]|' + # not ] or \\
+                        r'\\.|' +     # or an escaped character
+                        r'\](?!\])' + # ] not followed by ]
+                        ')*') +
+                       r'\]\])')      # ]]
+apertium_superblank_pat = (r'(?:\[' +     # [
+                           ('(?:' +
+                            r'[^\]\\]|' + # not ] or \\
+                            r'\\.' +      # or an escaped character
+                            ')*') +
+                           r'\])')        # ]
+apertium_blank_pat = ('(?:' +
+                      r'[^\[\]\\^]|' + # not []\^
+                      apertium_superblank_pat +
+                      ')*' +
+                      apertium_wblank_pat +
+                      '?')
+apertium_blank_regex = re.compile(apertium_blank_pat)
+
+def sort_analyses(instr):
+    ret = ''
+    s = instr
+    while s:
+        m = apertium_blank_regex.match(s)
+        ret += s[:m.end()]
+        s = s[m.end():]
+        if s and s[0] == '^':
+            pieces = []
+            last = 0
+            esc = False
+            for i in range(len(s)):
+                if esc:
+                    esc = False
+                    continue
+                elif s[i] == '\\':
+                    esc = True
+                elif s[i] == '/':
+                    pieces.append(s[last:i])
+                    last = i+1
+                elif s[i] == '$':
+                    pieces.append(s[last:i])
+                    s = s[i+1:]
+                    break
+            else:
+                ret += s
+                s = ''
+                break
+            ret += pieces[0]
+            if len(pieces) > 1:
+                ret += '/'
+                ret += '/'.join(sorted(pieces[1:]))
+            ret += '$'
+    ret += s # if something goes wrong, return the rest of the string as-is
+    return ret
+
 def run_command(cmd, intxt, outfile, shell=False):
     proc = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE, shell=shell)
@@ -176,7 +235,9 @@ class Step:
         'rtx-proc': 'transfer',
         'apertium-anaphora': 'anaph',
         'cg-conv': 'convert',
-        'vislcg3': 'disam'
+        'vislcg3': 'disam',
+        'apertium-extract-caps': 'decase',
+        'apertium-restore-caps': 'recase'
     }
     morphmodes = {
         '-b': 'biltrans',
@@ -203,7 +264,9 @@ class Step:
             self.args.pop(i)
         self.name = xml.attrib.get('debug-suff', 'unknown')
         if self.name == 'unknown':
-            if self.prog in Step.prognames:
+            if self.prog == 'lsx-proc' and '-p' in self.args:
+                self.name = 'postgen'
+            elif self.prog in Step.prognames:
                 self.name = Step.prognames[self.prog]
             elif self.prog in ['lt-proc', 'hfst-proc']:
                 self.name = 'morph'
@@ -355,6 +418,12 @@ class Corpus:
         self.command_list = ['all']
         if self.mode:
             self.command_list = Mode.all_modes[self.mode].get_commands()
+        self.sort = []
+        if 'sort' in blob:
+            if isinstance(blob['sort'], list):
+                self.sort = blob['sort']
+            elif blob['sort']:
+                self.sort = self.command_list
         self.commands = {c:i for i, c in enumerate(self.command_list)}
         self.relevant_commands = blob.get('relevant', [self.command_list[-1]])
         if not isinstance(self.relevant_commands, list):
@@ -414,7 +483,8 @@ class Corpus:
         }
         for c in self.command_list:
             expfile = self.exp_name(c)
-            outdata = load_output(self.out_name(c))
+            outdata = load_output(self.out_name(c),
+                                  should_sort_analyses=(c in self.sort))
             expdata = {}
             if os.path.isfile(expfile):
                 expdata = load_output(expfile)
@@ -976,7 +1046,7 @@ def check_hash(corpus, hsh):
                 expect = False
     return expect, gold
 
-def static_test(ignore_add=False, threshold=100):
+def static_test(ignore_add=False, threshold=100, quiet=True):
     n = len(Corpus.all_corpora.items())
     changed = set()
     total_tests = 0
@@ -1018,8 +1088,26 @@ def static_test(ignore_add=False, threshold=100):
             print('')
         print('')
     if changed:
-        print('There were changes! Rerun in interactive mode to update tests.')
-        print('Changed corpora: ' + ', '.join(sorted(changed)))
+        if quiet:
+            print('There were changes! Run `apertium-regtest cli` to update tests.')
+            print('Changed corpora: ' + ', '.join(sorted(changed)))
+        else:
+            print('There were changes!')
+            print('The tests need to be updated.')
+            ls = sorted(changed)
+            print('The corpora that contain changes are:', ', '.join(ls))
+            print('This can be done in a browser by running')
+            print('')
+            print('  apertium-regtest web')
+            print('')
+            print('or on the command line with')
+            print('')
+            print('  apertium-regtest cli')
+            print('')
+            print('A specific corpus can be edited with `-c`, for example')
+            print('')
+            print('  apertium-regtest -c %s web' % ls[0])
+            print('')
     else:
         print('All tests pass.')
     return ((100.0 * total_passes) / total_tests) >= threshold
@@ -1056,6 +1144,10 @@ apertium-regtest has 3 modes available:
         default_min = int(os.environ['AP_REGTEST_MIN'])
     test_gp.add_argument('-t', '--threshold', type=int, default=default_min,
                          help="percentage of tests required to count as passing (default 100 or AP_REGTEST_MIN)")
+    default_quiet = os.environ.get('AP_REGTEST_QUIET', 'no').lower() == 'yes'
+    test_gp.add_argument('-q', '--quiet', action='store_true',
+                         help="print minimal error message on test failure",
+                         default=default_quiet)
 
     # WEB ARGUMENTS
     web_gp = parser.add_argument_group('web mode options')
@@ -1080,7 +1172,8 @@ apertium-regtest has 3 modes available:
     if args.mode == 'test':
         load_corpora(args.corpus, static=True)
         try:
-            if not static_test(args.ignore_add, threshold=args.threshold):
+            if not static_test(args.ignore_add, threshold=args.threshold,
+                               quiet=args.quiet):
                 sys.exit(1)
         except (InputFileDoesNotExist, InputFileIsEmpty, ErrorInPipeline):
             sys.exit(1)
